package com.qmth.distributed.print.business.activiti.custom.listener;

import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;
import com.qmth.distributed.print.business.entity.ExamTask;
import com.qmth.distributed.print.business.entity.TFFlowLog;
import com.qmth.distributed.print.business.enums.CustomFlowMultipleUserApproveTypeEnum;
import com.qmth.distributed.print.business.enums.MessageEnum;
import com.qmth.distributed.print.business.service.BasicMessageService;
import com.qmth.distributed.print.business.service.PrintCommonService;
import com.qmth.teachcloud.common.bean.params.ApproveUserResult;
import com.qmth.teachcloud.common.contant.SpringContextHolder;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.enums.FlowApproveOperationEnum;
import com.qmth.teachcloud.common.enums.TFCustomTypeEnum;
import com.qmth.teachcloud.common.service.SysUserService;
import com.qmth.teachcloud.common.util.JacksonUtil;
import org.activiti.engine.HistoryService;
import org.activiti.engine.TaskService;
import org.activiti.engine.delegate.event.ActivitiEvent;
import org.activiti.engine.delegate.event.ActivitiEventListener;
import org.activiti.engine.delegate.event.ActivitiProcessStartedEvent;
import org.activiti.engine.history.HistoricVariableInstance;
import org.activiti.engine.task.Task;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.io.Serializable;
import java.util.*;

/**
 * @Description: 全局监听
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2022/2/8
 */
@Component
public class ProcessEventListener implements ActivitiEventListener, Serializable {
    private final static Logger log = LoggerFactory.getLogger(ProcessEventListener.class);

    @Override
    public void onEvent(ActivitiEvent event) {
        switch (event.getType()) {
            case PROCESS_STARTED:
                // 流程启动
                log.info("流程启动_PROCESS_STARTED:ProcessInstanceId:{},ExecutionId:{},ProcessDefinitionId:{}", event.getProcessInstanceId(), event.getExecutionId(), event.getProcessDefinitionId());
                BasicMessageService basicMessageService = SpringContextHolder.getBean(BasicMessageService.class);
                ActivitiProcessStartedEvent activitiProcessStartedEvent = (ActivitiProcessStartedEvent) event;
                Map<String, Object> map = activitiProcessStartedEvent.getVariables();
                Long objectId = (Long) map.get(SystemConstant.OBJECT_ID);
                TFCustomTypeEnum flowType = (TFCustomTypeEnum) map.get(SystemConstant.FLOW_TYPE);
                Object o = map.get(SystemConstant.OBJECT_DATA);
                Boolean flowSubmit = (Boolean) map.get(SystemConstant.FLOW_SUBMIT);
                //发送命题待审批短信
                if (Objects.nonNull(objectId) && Objects.nonNull(flowType) && Objects.nonNull(o) && Objects.nonNull(flowSubmit) && flowSubmit) {
                    if (flowType == TFCustomTypeEnum.ELECTRON_FLOW && o instanceof ExamTask) {
                        basicMessageService.sendNoticeTaskCreate((ExamTask) o);
                    }
                }
                break;
            case PROCESS_COMPLETED:
                // 流程结束
                log.info("流程结束_PROCESS_COMPLETED:ProcessInstanceId:{},ExecutionId:{},ProcessDefinitionId:{}", event.getProcessInstanceId(), event.getExecutionId(), event.getProcessDefinitionId());
                HistoryService historyService = SpringContextHolder.getBean(HistoryService.class);
                SysUserService sysUserService = SpringContextHolder.getBean(SysUserService.class);
                PrintCommonService printCommonService = SpringContextHolder.getBean(PrintCommonService.class);

                List<HistoricVariableInstance> historicVariableInstanceList = historyService.createHistoricVariableInstanceQuery().processInstanceId(event.getProcessInstanceId()).list();
                if (Objects.nonNull(historicVariableInstanceList) && historicVariableInstanceList.size() > 0) {
                    TFFlowLog tfFlowLog = null;
                    Object object = null;
                    for (HistoricVariableInstance h : historicVariableInstanceList) {
                        if (Objects.equals(h.getVariableName(), SystemConstant.APPROVE_TF_FLOW_LOG)) {
                            tfFlowLog = (TFFlowLog) h.getValue();
                            Optional.ofNullable(tfFlowLog).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("流程审批日志为空"));
                        } else if (Objects.equals(h.getVariableName(), SystemConstant.OBJECT_DATA)) {
                            object = h.getValue();
                            Optional.ofNullable(object).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("实体数据为空"));
                        }
                        if (Objects.nonNull(object) && Objects.nonNull(tfFlowLog)) {
                            break;
                        }
                    }
                    if (Objects.nonNull(tfFlowLog.getObjectTable())) {
                        if (Objects.equals(tfFlowLog.getObjectTable(), TFCustomTypeEnum.ELECTRON_FLOW.getTable())) {//如果是命题任务交卷
                            //取命题老师ID
                            SysUser sysUser = sysUserService.getById(((ExamTask) object).getUserId());
                            try {
                                printCommonService.checkData(((ExamTask) object).getSchoolId(), ((ExamTask) object).getCourseCode(), ((ExamTask) object).getPaperNumber(), sysUser);
                            } catch (IOException e) {
                                throw ExceptionResultEnum.ERROR.exception("生成pdf失败");
                            }
                        }
                    }
                }
                break;
//            case ACTIVITY_COMPENSATE:
//                // 一个节点将要被补偿。事件包含了将要执行补偿的节点id。
//                log.info("ACTIVITY_COMPENSATE:ProcessInstanceId:{},ExecutionId:{},ProcessDefinitionId:{}", event.getProcessInstanceId(), event.getExecutionId(), event.getProcessDefinitionId());
//                break;
//            case ACTIVITY_COMPLETED:
//                // 一个节点成功结束
//                log.info("流程节点完成_ACTIVITY_COMPLETED:ProcessInstanceId:{},ExecutionId:{},ProcessDefinitionId:{}", event.getProcessInstanceId(), event.getExecutionId(), event.getProcessDefinitionId());
//                break;
//            case ACTIVITY_ERROR_RECEIVED:
//                // 一个节点收到了一个错误事件。在节点实际处理错误之前触发。 事件的activityId对应着处理错误的节点。 这个事件后续会是ACTIVITY_SIGNALLED或ACTIVITY_COMPLETE， 如果错误发送成功的话。
//                log.info("ACTIVITY_ERROR_RECEIVED:ProcessInstanceId:{},ExecutionId:{},ProcessDefinitionId:{}", event.getProcessInstanceId(), event.getExecutionId(), event.getProcessDefinitionId());
//                break;
//            case ACTIVITY_MESSAGE_RECEIVED:
//                // 一个节点收到了一个消息。在节点收到消息之前触发。收到后，会触发ACTIVITY_SIGNAL或ACTIVITY_STARTED，这会根据节点的类型（边界事件，事件子流程开始事件）
//                log.info("ACTIVITY_MESSAGE_RECEIVED:ProcessInstanceId:{},ExecutionId:{},ProcessDefinitionId:{}", event.getProcessInstanceId(), event.getExecutionId(), event.getProcessDefinitionId());
//                break;
//            case ACTIVITY_SIGNALED:
//                // 一个节点收到了一个信号
//                log.info("ACTIVITY_SIGNALED:ProcessInstanceId:{},ExecutionId:{},ProcessDefinitionId:{}", event.getProcessInstanceId(), event.getExecutionId(), event.getProcessDefinitionId());
//                break;
//            case ACTIVITY_STARTED:
//                // 一个节点开始执行
//                log.info("ACTIVITY_STARTED:ProcessInstanceId:{},ExecutionId:{},ProcessDefinitionId:{}", event.getProcessInstanceId(), event.getExecutionId(), event.getProcessDefinitionId());
//                break;
//            case CUSTOM:
//                log.info("CUSTOM:ProcessInstanceId:{},ExecutionId:{},ProcessDefinitionId:{}", event.getProcessInstanceId(), event.getExecutionId(), event.getProcessDefinitionId());
//                break;
//            case ENGINE_CLOSED:
//                // 监听器监听的流程引擎已经关闭，不再接受API调用。
//                log.info("ENGINE_CLOSED:ProcessInstanceId:{},ExecutionId:{},ProcessDefinitionId:{}", event.getProcessInstanceId(), event.getExecutionId(), event.getProcessDefinitionId());
//                break;
//            case ENGINE_CREATED:
//                // 监听器监听的流程引擎已经创建完毕，并准备好接受API调用。
//                log.info("ENGINE_CREATED:ProcessInstanceId:{},ExecutionId:{},ProcessDefinitionId:{}", event.getProcessInstanceId(), event.getExecutionId(), event.getProcessDefinitionId());
//                break;
//            case ENTITY_ACTIVATED:
//                // 激活了已存在的实体，实体包含在事件中。会被ProcessDefinitions, ProcessInstances 和 Tasks抛出。
//                log.info("ENTITY_ACTIVATED:ProcessInstanceId:{},ExecutionId:{},ProcessDefinitionId:{}", event.getProcessInstanceId(), event.getExecutionId(), event.getProcessDefinitionId());
//                break;
//            case ENTITY_CREATED:
//                // 创建了一个新实体。实体包含在事件中。
//                log.info("ENTITY_CREATED:ProcessInstanceId:{},ExecutionId:{},ProcessDefinitionId:{}", event.getProcessInstanceId(), event.getExecutionId(), event.getProcessDefinitionId());
//                break;
//            case ENTITY_DELETED:
//                // 删除了已存在的实体。实体包含在事件中
//                log.info("ENTITY_DELETED:ProcessInstanceId:{},ExecutionId:{},ProcessDefinitionId:{}", event.getProcessInstanceId(), event.getExecutionId(), event.getProcessDefinitionId());
//                break;
//            case ENTITY_INITIALIZED:
//                // 创建了一个新实体，初始化也完成了。如果这个实体的创建会包含子实体的创建，这个事件会在子实体都创建/初始化完成后被触发，这是与ENTITY_CREATED的区别。
//                log.info("ENTITY_INITIALIZED:ProcessInstanceId:{},ExecutionId:{},ProcessDefinitionId:{}", event.getProcessInstanceId(), event.getExecutionId(), event.getProcessDefinitionId());
//                break;
//            case ENTITY_SUSPENDED:
//                // 暂停了已存在的实体。实体包含在事件中。会被ProcessDefinitions, ProcessInstances 和 Tasks抛出。
//                log.info("ENTITY_SUSPENDED:ProcessInstanceId:{},ExecutionId:{},ProcessDefinitionId:{}", event.getProcessInstanceId(), event.getExecutionId(), event.getProcessDefinitionId());
//                break;
//            case ENTITY_UPDATED:
//                // 更新了已存在的实体。实体包含在事件中。
//                log.info("ENTITY_UPDATED:ProcessInstanceId:{},ExecutionId:{},ProcessDefinitionId:{}", event.getProcessInstanceId(), event.getExecutionId(), event.getProcessDefinitionId());
//                break;
//            case JOB_EXECUTION_FAILURE:
//                // 作业执行失败。作业和异常信息包含在事件中。
//                log.info("JOB_EXECUTION_FAILURE:ProcessInstanceId:{},ExecutionId:{},ProcessDefinitionId:{}", event.getProcessInstanceId(), event.getExecutionId(), event.getProcessDefinitionId());
//                break;
//            case JOB_EXECUTION_SUCCESS:
//                // 作业执行成功。job包含在事件中。
//                log.info("JOB_EXECUTION_SUCCESS:ProcessInstanceId:{},ExecutionId:{},ProcessDefinitionId:{}", event.getProcessInstanceId(), event.getExecutionId(), event.getProcessDefinitionId());
//                break;
//            case JOB_RETRIES_DECREMENTED:
//                // 因为作业执行失败，导致重试次数减少。作业包含在事件中。
//                log.info("JOB_RETRIES_DECREMENTED:ProcessInstanceId:{},ExecutionId:{},ProcessDefinitionId:{}", event.getProcessInstanceId(), event.getExecutionId(), event.getProcessDefinitionId());
//                break;
//            case MEMBERSHIPS_DELETED:
//                // 所有成员被从一个组中删除。在成员删除之前触发这个事件，所以他们都是可以访问的。 因为性能方面的考虑，不会为每个成员触发单独的MEMBERSHIP_DELETED事件。
//                log.info("MEMBERSHIPS_DELETED:ProcessInstanceId:{},ExecutionId:{},ProcessDefinitionId:{}", event.getProcessInstanceId(), event.getExecutionId(), event.getProcessDefinitionId());
//                break;
//            case MEMBERSHIP_CREATED:
//                // 用户被添加到一个组里。事件包含了用户和组的id。
//                log.info("MEMBERSHIP_CREATED:ProcessInstanceId:{},ExecutionId:{},ProcessDefinitionId:{}", event.getProcessInstanceId(), event.getExecutionId(), event.getProcessDefinitionId());
//                break;
//            case MEMBERSHIP_DELETED:
//                // 用户被从一个组中删除。事件包含了用户和组的id。
//                log.info("MEMBERSHIP_DELETED:ProcessInstanceId:{},ExecutionId:{},ProcessDefinitionId:{}", event.getProcessInstanceId(), event.getExecutionId(), event.getProcessDefinitionId());
//                break;
//            case TASK_ASSIGNED:
//                // 任务被分配给了一个人员。事件包含任务。
//                log.info("TASK_ASSIGNED:ProcessInstanceId:{},ExecutionId:{},ProcessDefinitionId:{}", event.getProcessInstanceId(), event.getExecutionId(), event.getProcessDefinitionId());
//                break;
            case TASK_COMPLETED:
                // 任务被完成了。它会在ENTITY_DELETE事件之前触发。当任务是流程一部分时，事件会在流程继续运行之前， 后续事件将是ACTIVITY_COMPLETE，对应着完成任务的节点。
                log.info("流程任务完成_TASK_COMPLETED:ProcessInstanceId:{},ExecutionId:{},ProcessDefinitionId:{}", event.getProcessInstanceId(), event.getExecutionId(), event.getProcessDefinitionId());
                TaskService taskService = SpringContextHolder.getBean(TaskService.class);
                sysUserService = SpringContextHolder.getBean(SysUserService.class);
                basicMessageService = SpringContextHolder.getBean(BasicMessageService.class);

                Task task = taskService.createTaskQuery().executionId(event.getExecutionId()).singleResult();
                if (Objects.nonNull(task)) {
                    TFFlowLog tfFlowLog = (TFFlowLog) taskService.getVariable(task.getId(), SystemConstant.APPROVE_TF_FLOW_LOG);
                    Optional.ofNullable(tfFlowLog).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("流程审批日志为空"));
                    o = taskService.getVariable(task.getId(), SystemConstant.OBJECT_DATA);
                    Optional.ofNullable(o).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("实体数据为空"));
                    if (tfFlowLog.getApproveOperation() != FlowApproveOperationEnum.SUBMIT
                            && tfFlowLog.getApproveOperation() != FlowApproveOperationEnum.END
                            && tfFlowLog.getApproveOperation() != FlowApproveOperationEnum.FINISH) {
                        if (Objects.nonNull(tfFlowLog.getPendApproveId())) {
                            String[] strs = tfFlowLog.getPendApproveId().split(",");
                            List<Long> userIds = new ArrayList<>();
                            for (String s : strs) {
                                userIds.add(SystemConstant.convertIdToLong(s.trim()));
                            }
                            List<SysUser> sysUsers = sysUserService.listByIds(userIds);
                            List<ApproveUserResult> sysUserList = new Gson().fromJson(JacksonUtil.parseJson(sysUsers), new TypeToken<List<ApproveUserResult>>() {
                            }.getType());

                            List<ApproveUserResult> nextApproverPeopleList = new ArrayList<>();
                            nextApproverPeopleList.addAll(sysUserList);
                            if (Objects.nonNull(tfFlowLog.getApproveUserApproveType()) && tfFlowLog.getApproveUserApproveType() == CustomFlowMultipleUserApproveTypeEnum.ORDER) {
                                nextApproverPeopleList.clear();
                                nextApproverPeopleList.add(sysUserList.get(0));
                            }

                            if (Objects.nonNull(nextApproverPeopleList) && (tfFlowLog.getApproveOperation() == FlowApproveOperationEnum.APPROVE
                                    || tfFlowLog.getApproveOperation() == FlowApproveOperationEnum.EXCHANGE)) {
                                //如果不是并行会签，则每个节点都发
                                if (Objects.nonNull(tfFlowLog.getApproveUserApproveType()) && tfFlowLog.getApproveUserApproveType() != CustomFlowMultipleUserApproveTypeEnum.ALL) {
                                    if (Objects.nonNull(o) && o instanceof ExamTask) {
                                        basicMessageService.sendNoticeTaskAuditFlow((ExamTask) o, nextApproverPeopleList, MessageEnum.NOTICE_OF_AUDIT_CREATED);
                                    }
                                } else {
                                    if (Objects.isNull(tfFlowLog.getApproveMultiSetup()) || tfFlowLog.getApproveMultiSetup().intValue() == 0) {
                                        if (Objects.nonNull(o) && o instanceof ExamTask) {
                                            basicMessageService.sendNoticeTaskAuditFlow((ExamTask) o, nextApproverPeopleList, MessageEnum.NOTICE_OF_AUDIT_CREATED);
                                        }
                                    }
                                }
                            } else if (Objects.nonNull(nextApproverPeopleList) && (tfFlowLog.getApproveOperation() == FlowApproveOperationEnum.REJECT
                                    || tfFlowLog.getApproveOperation() == FlowApproveOperationEnum.CANCEL)) {
                                if (Objects.nonNull(o) && o instanceof ExamTask) {
                                    basicMessageService.sendNoticeTaskAuditFlow((ExamTask) o, nextApproverPeopleList, MessageEnum.NOTICE_OF_AUDIT_REJECT);
                                }
                            }
                        }
                    }
                }
                break;
//            case TIMER_FIRED:
//                // 触发了定时器。job包含在事件中。
//                log.info("TIMER_FIRED:ProcessInstanceId:{},ExecutionId:{},ProcessDefinitionId:{}", event.getProcessInstanceId(), event.getExecutionId(), event.getProcessDefinitionId());
//                break;
//            case UNCAUGHT_BPMN_ERROR:
//                log.info("UNCAUGHT_BPMN_ERROR:ProcessInstanceId:{},ExecutionId:{},ProcessDefinitionId:{}", event.getProcessInstanceId(), event.getExecutionId(), event.getProcessDefinitionId());
//                break;
//            case VARIABLE_CREATED:
//                log.info("VARIABLE_CREATED:ProcessInstanceId:{},ExecutionId:{},ProcessDefinitionId:{}", event.getProcessInstanceId(), event.getExecutionId(), event.getProcessDefinitionId());
//                break;
//            case VARIABLE_DELETED:
//                log.info("VARIABLE_DELETED:ProcessInstanceId:{},ExecutionId:{},ProcessDefinitionId:{}", event.getProcessInstanceId(), event.getExecutionId(), event.getProcessDefinitionId());
//                break;
//            case VARIABLE_UPDATED:
//                log.info("VARIABLE_UPDATED:ProcessInstanceId:{},ExecutionId:{},ProcessDefinitionId:{}", event.getProcessInstanceId(), event.getExecutionId(), event.getProcessDefinitionId());
//                break;
            default:
//                log.info("default->type:{},ProcessInstanceId:{},ExecutionId:{},ProcessDefinitionId:{}", event.getType(), event.getProcessInstanceId(), event.getExecutionId(), event.getProcessDefinitionId());
                break;
        }
    }

    @Override
    public boolean isFailOnException() {
        return false;
    }
}
